/* Arduino Projects for Dummies
 * by Brock Craft 
 *
 * Chapter 6: Making a Scrolling Sign
 * Creates sprites and text messages using an
 * 8x8 LED matrix display 
 *
 * This sketch sends a string of text to the display
 *
 * v0.1 30.04.2013
 * Adapted from Oomlout.com http://www.tinyurl.com/yhwxv6h
*/
#include "alphabit.h" // Ajout version FR OE

char message[] = "12345678";  // Le message à épeler

// Variables used for scrolling (both start at 0)
int index  = 0;  // This is the current signe in the string being displayed
int offset = 0;  // This is how many columns it is offset by

// Variables defining each signes position in an array of integer arrays
// lettres
// const int A = 0;  const int B = 1;  etc.
// Punctuation
// const int COL =26; const int DASH = 27; etc.
// Extra Characters
// const int  FULL =30; const int CHECK = 31; etc.

// Pin Definitions
//int broLig[] = {2,3,4,5,6,7,8,9};         // Lignes anodes (actif=HAUT))
int broLig[] = { 2,3,4,5,6,7,8,9,};         // Lignes anodes (actif=HAUT))
int broCol[] = {17,16,15,14,13,12,11,10}; // Colonnes cathodes communes (actif=BAS))

// The array used to hold a bitmap of the display 
// (if you wish to do something other than a scrolling marquee 
// change the data in this variable)
byte data[] = {0,0,0,0,0,0,0,0};        

//The alphabet
//Each signe is an 8 x 7 bitmap where 1 is on and 0 is off
// const int _A[] = {B0001000, etc.

// Load the bitmap characters into an array (each character's position corresponds to its previously defined index 
// (ie _A (a's bitmap) is at index 0 and A = 0 so lettres[A] will return the 'A' bitmap.)
const int * lettres[] = {_A,_B,_C,_D,_E,_F,_G,_H,_I,_J,_K,_L,_M,_N,_O,_P,_Q,_R,_S,_T,_U,_V,_W,_X,_Y,_Z,
_COL,_DASH,_BRA2,__, _FULL, _CHECK, _B2, _TEMP, _LINE, _SMILE, _DOT, _COLDOT, _TSTL1, _TSTL2, _TSTL3, _TSTL4};

// An array holding the puiss2 of 2. These are used as bit masks when calculating what to display
const int puiss2[] = {1,2,4,8,16,32,64,128};

// Setup runs once when power is applied
void setup() { 
  for(int i = 0; i <8; i++){  // Configure les 16 broches en sortie
    pinMode(broLig[i], OUTPUT);
    pinMode(broCol[i], OUTPUT);
  }
}
 
void loop() {
  chargerLutin();
  afficherLutin(data, 50);  // Délai
}

// Loads the current scroll state frame into the data[] display array
void chargerLutin() {
  int charAc = lireSigne(message[index]);
  int charSu = lireSigne(message[index+1]);
  
  for(int li=0; li < 8; li++){                    // Iterate through each row
    data[li] = 0;                                   // Reset the row we're working on
    for(int co=0; co < 8; co++){         // Iterate through each column
     data[li] = data[li] + ((puiss2[co] & (lettres[charAc][li] <<    offset) ));   //loads the current charachter offset by offset pixels 
     data[li] = data[li] +  (puiss2[co] & (lettres[charSu][li] >> (8-offset) ));   //loads the next charachter offset by offset pixels
    }
  }
  offset++;                                          // Increment the offset by one row
  if (offset==8) {
    offset = 0;
    index++; 
    if (index==sizeof(message)-2) { index=0; }
  }    //if offset is 8 load the next character pair for the next time through
}

void afficherLutin(byte * data, unsigned long duree) {
  unsigned long start = millis();
  while (start+duree > millis()) {
    for (int cof = 0; cof < 8; cof++) {  // Iterate through each column
      for (int i = 0; i < 8; i++) {                         
         digitalWrite(broLig[i], LOW);  // Turn off all row pins  
      }
      for (int i = 0; i < 8; i++) {        // Set only the one pin
        if (i == cof) {
          digitalWrite(broCol[i], LOW);
        }                                // Turns the current row on
        else {
          digitalWrite(broCol[i], HIGH); } // Turns the rest of the rows off
        }
      for (int lif = 0; lif < 8; lif++) {  // Iterate through each pixel in the current column
        int bit = (data[cof] >> lif) & 1;
        if (bit == 1) { 
          digitalWrite(broLig[lif], HIGH); // If the bit in the data array is set, turn the LED on
        }
      }
    } 
  }
}

