// APPN08_Verrou
/* Auteur: Brock Craft 
 *
 * Verrou de porte électrique
 * Saisie du code avec clavier et affichage LED quadruple
 * piloté par un MAX72xx.
 *
 * v0.1 30.04.2013
 * Adapté de la technique clavier de Michael Margolis
 * VF par OE 160625
*/

#include <SPI.h>

const int nbrChiffres = 4;    // Nombre de chiffres de l'afficheur
const int nbrClaLig = 4;      // Nombre de lignes du clavier
const int nbrClaCol = 3;      // Nombre de colonnes du clavier
const int antiRebond = 20;    // Nombre de ms de filtrage des rebonds de contact
const int dureeDeverr = 5000; // Durée d'ouverture du verrou

const int broDeverr = 9;      // Commande du relais
const int broSlave = 10;      // Commande du mode esclave du MAX72xx

char code[4]= {'1','2','3','4'}; // ECRIVEZ LE CODE SECRET ICI
char codeSaisi[4];               // Stocke le code saisi

boolean DEBUG=true;  // Si true, affiche dans le moniteur série
int nbrFrappes=0;    // Nombre de touches détectées

// Matrice des touches du clavier
const char planClavier[nbrClaLig][nbrClaCol] = {
  { '1', '2', '3' },
  { '4', '5', '6' },
  { '7', '8', '9' },
  { '*', '0', '#' }
};

// Broches numériques Arduino pour les lignes et colonnes
// du clavier Sparkfun ou Rapid.
const int broLig[nbrClaLig] = { 7, 2, 3, 5  }; // Lignes 0 à 3
const int broCol[nbrClaCol] = { 6, 8 ,4 };     // Colonnes 0 à 2
// Variante pour clavier Ada 419
// const int broLig[nbrClaLig] = {8, 7, 6, 5};
// const int broCol[nbrClaCol] = {4, 3, 2};

void setup()
{
  if (DEBUG){Serial.begin(9600);} // Active le port série si DEBUG mode

  // Configure le MAX72xx pour format 7 segments (voir datasheet)
  // Format: commanderAff(commande, valeur)
  
  SPI.begin();   // Demarre interface SPI pour le MAX72xx
  pinMode(broSlave, OUTPUT);   // Broche SPI en sortie
  commanderAff(12,1);  // Mode normal (mode shutdown par défaut)
  commanderAff(15,0);  // Arrêt test afficheur
  commanderAff(10,8);  // Luminosité moyenne (plage 0-15)
  commanderAff(11, nbrChiffres);  // Limite de scan 7221
  commanderAff(9,255); // Active mode "Decode qui gère 0-9, H, E, L, P, -, vide 

  // Config des broches clavier et relais
  pinMode(broDeverr, OUTPUT);
  
  for (int vLig = 0; vLig < nbrClaLig; vLig++)
  {
    pinMode(broLig[vLig], INPUT);       // Broches vLig en entrée
    digitalWrite(broLig[vLig], HIGH);   // Active rés. de rappel internes
  }
  for (int vCol = 0; vCol < nbrClaCol; vCol++)
  {
    pinMode(broCol[vCol], OUTPUT);     // Broches vCol en sortie
    digitalWrite(broCol[vCol], HIGH);
  }
  // Vidage afficheur (utile au débogage)
  effacerAff();                        
}

void loop()
{
  char touche = lireTouche();     // Si touche frappée, stockage...
  codeSaisi[nbrFrappes] = touche; // dans le tableau (buffer)

  if( touche != 0) {       // Si différent de 0, affichage
    commanderAff(nbrFrappes+1, touche); 
    // Section réservée à la mise au point
    if (DEBUG){
      Serial.print("Chiffre ");
      Serial.println(nbrFrappes);
      Serial.print("Touche lue: ");
      Serial.println(touche);
      Serial.print("Essai: ");
      Serial.print(codeSaisi[0]); // Affichage du code saisi
      Serial.print(codeSaisi[1]);
      Serial.print(codeSaisi[2]);
      Serial.println(codeSaisi[3]);
      Serial.print("Code: ");     // Affichage du code secret
      Serial.print(code[0]);
      Serial.print(code[1]);
      Serial.print(code[2]);
      Serial.println(code[3]);
    }

    nbrFrappes++;          // Compteur de frappes
    if (nbrFrappes == 4){  // Comparer avec code secret
      delay(500);          // Attendre affichage du 4eme chiffre
      if (memcmp(codeSaisi, code, 4) == 0) {  // Code bon ?
        if(DEBUG) {Serial.println("MATCH!");} // Si DEBUG, annoncer
        deverrouiller();     // Déverrouiller
      } 
      effacerAff();            // Vider l'affichage
      memset(codeSaisi, 0, 4); // Vider le tableau tampon
      nbrFrappes=0;
    }
  }
}

// Renvoie le chiffre frappé ou 0 si rien
char lireTouche()
{
  char touche = 0;                                // 0 = aucune touche
  for(int vCol = 0; vCol < nbrClaCol; vCol++)
  {
    digitalWrite(broCol[vCol],LOW);               // Active la colonne
    for(int vLig = 0; vLig < nbrClaLig; vLig++)   // Scrute les lignes
    {
      if(digitalRead(broLig[vLig]) == LOW)        // Si frappe...
      {
        delay(antiRebond);                        // Anti-rebond
        while(digitalRead(broLig[vLig]) == LOW) ; // Attend que touche relâchée
        touche = planClavier[vLig][vCol];         // Stocke le chiffre trouvé
      }
    }
    digitalWrite(broCol[vCol],HIGH);              // Désactive la colonne
  }
  return touche;  // Renvoie la touche frappée ou 0
}

void commanderAff(int commande, unsigned char valeur)
{
  digitalWrite(broSlave, LOW);   // CS actif quand broche LOW
  SPI.transfer(commande);        // 1er octet de commande vers MAX72xx
  SPI.transfer(valeur);          // 2nd octet de données vers MAX72xx
  digitalWrite(broSlave, HIGH);  // Fin d'envoi, désélection du MAX
}  

void effacerAff(){
  commanderAff(1, '_'); // Efface l'affichage
  commanderAff(2, '_'); 
  commanderAff(3, '_');
  commanderAff(4, '_'); 
}

void deverrouiller(){  
  if(DEBUG) {Serial.println("Porte ouverte."); }

    commanderAff(1, 'o'); // espace
    commanderAff(2, 'l');
    commanderAff(3, '1');
    commanderAff(4, 'o');
    digitalWrite(broDeverr,HIGH);  // Active le relais pour déverrouiller
    delay(dureeDeverr);            // Pause pour passer
    digitalWrite(broDeverr,LOW);   // Désactive le relais pour reverrouiller
}

